#############################################################################
# Visual Tcl v1.11p1 Project
#

#################################
# GLOBAL VARIABLES
#
global nodats; 
global outstyle; 
global widget; 
#RPC
#Added infile/outfile reference
global infile; 
global outfile; 
###

#################################
# USER DEFINED PROCEDURES
#
proc init {argc argv} {
global nodats
    set nodats 0
}

init $argc $argv


proc {cleanString} {str} {
#
    regsub -all {\(} $str "" str
    regsub -all {\)} $str "" str
    regsub -all {\[} $str "" str
    regsub -all {\]} $str "" str
    regsub -all {\$} $str "" str
    regsub -all {\.} $str "" str
    regsub -all {\*} $str "" str
    regsub -all {\^} $str "" str
    regsub -all {\?} $str "" str
    regsub -all {\'} $str "" str
    regsub -all {\:} $str "" str
    regsub -all {\-} $str "" str
    regsub -all {\!} $str "" str
    regsub -all {\,} $str "" str
    return $str
}

proc {convertFile} {} {
#
#RPC
#Added infile/outfile reference
#Added infile/outfile if statements
    global infile
    global outfile

    if {$infile == ""} {
        set itypes {
            {{Windeck2 Decks} {.deck} }
            {{Windeck1 Decks} {.wdi} }
            {{All Files}       *      }
        }
        set otypes {
            {{CardTable Decks} {.ctd} }
            {{All Files}       *      }
        }
        set file [tk_getOpenFile -defaultextension "deck" -filetypes $itypes -parent .top17 -title "Open Deck"]
    } else {
        set file $infile
    }
    if {$file != ""} {
        if {$outfile == ""} {
            if {[regexp -nocase "(.*)\\.deck" $file matchStr deckName]} {
                set ofile "${deckName}.ctd"
            } else {
                set ofile "cardtable.ctd"
            }
            set ofile [tk_getSaveFile -defaultextension "ctd" -initialfile $ofile -filetypes $otypes -parent .top17 -title "Save Deck As"]
        } else {
            set ofile $outfile
        }
        if {$ofile != ""} {
            wdConvert $file $ofile
        }
    }
###
}

proc {eq} {str} {
regsub -all {"} $str {\"} str
    regsub -all {\|} $str {;} str
    return $str
}

proc {eq0} {str} {
if {[string index $str 0] == "\""} {
        set str "\\$str"
    }
    while {[regexp {(.*)([^\\])(".*)} $str matchVar s1 c s2]} {
        set str "${s1}${c}\\${s2}"
    }
    return $str
}

proc {escapeChars} {str} {
regsub -all {\(} $str {\(} str
    regsub -all {\)} $str {\)} str
    regsub -all {\[} $str {\[} str
    regsub -all {\]} $str {\]} str
    regsub -all {\$} $str {\$} str
    regsub -all {\.} $str {\.} str
    regsub -all {\*} $str {\*} str
    regsub -all {\^} $str {\^} str
    regsub -all {\?} $str {\?} str
    return $str
}

proc {fixSet} {str} {
switch $str {
      p {return "pU"}
      U {return "pU"}
      h {return "hH"}
      H {return "hH"}
      a {return "aA"}
      A {return "aA"}
      d {return "dD"}
      D {return "dD"}
      default {return $str}
   }

   return $str
}

proc {makePad} {str} {
return "         "
    set string ""
    set padLen [expr 8 - (([string length $str])%9) ]
    for {set i 0} {$i < $padLen} {incr i} {
        append string "_"
    }
    return $string
}

proc {makeTitle} {name destiny stats icons} {
append title "[eq $name]"
    set destiny " ($destiny)"
    if {(9-[string length $title]%9) < [string length $destiny]} {
        append title [makePad $name]
    }
    append title "$destiny $stats"
    append title [showIcons $icons]
    return "$title\" "
}

proc {makeTitleThumb} {name set alignment} {
append title "$name"


    set cardname ""
    set cardnameadd ""

    append cardname "/starwars/"

    set setlist [split [showSet $set] " "]
    if {[llength $setlist] != 0} {
        foreach i $setlist {
            append cardname [cleanString $i]
        }
    }

    append cardname "-"
    append cardname [showSide $alignment]
    append cardname "/t_"

    set namelist [split $name " "]
    if {[llength $namelist] != 0} {
        foreach i $namelist {
            append cardname [cleanString $i]
        }
    }

    append cardname "\" "    

    return $cardname
}

proc {readData} {wd2dir} {
global data
    global data1
    global nodats

#RPC
#Added $wd2dir reference
    set datFile [glob -nocomplain "$wd2dir/*.dat"]
###

    if {! [llength $datFile] } {
        write_text "*** No datafiles were read.\n"
        write_text "Did you copy wd2ctd.tcl into the WinDeck data directory?.\n"
        write_text "wd2ctd.tcl should be run from the same directory that the data files (anh.dat, premiere.dat, etc.) are found.\n"
        write_text "Quit this program, copy it to that directory and try again.\n"
        set nodats 1
        return
    }

###RPC
###Removed all wd2dir references 
    foreach {file} $datFile {
        write_text "*** Reading ${file}\n"
        if {[file exists ${file}]} {
        write_text "\["

            set fp [open ${file} r]
            set i 0;
            while {! [eof $fp] } {
                set dataline [gets $fp]
                if {[regexp {^[ld];} $dataline]} {
                    if {$file == "cards.dat"} {
                        lappend data1 $dataline
                    } else {
                        lappend data $dataline
                    }

                    if {${i}%10 == 0} {
                        write_text "."
                    }
                    incr i
                } else {
#                    write_text "Error: $dataline\n"
                }
            }
            write_text "\]\n"
            close $fp
        } else {
            write_text "*** WD data ${file} does not exist.\n"
        }
    }
###
    lappend data {}
    lappend data1 {}
}

proc {semiToCard} {raw_card cardSet} {
set reStr {[^;]*}
  set reStrPlus {[^;]+}
  set reForce {[ld]}

set reCharacter "^($reForce);(c);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reCreature  "^($reForce);(r);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reDevice    "^($reForce);(d);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reEffect    "^($reForce);(e);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reEpicEvent "^($reForce);(E);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reInterrupt "^($reForce);(i);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reJediTest  "^($reForce);(j);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reLocation  "^($reForce);(l);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reStarship  "^($reForce);(s);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reVehicle   "^($reForce);(v);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
set reWeapon    "^($reForce);(w);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"

set reObjective "^($reForce);(O);(.);($reStrPlus);($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
#               force       type sub set          uniq     name     rarity   destiny  stats    icons    lore     text

  set c_class ""
  set c_destiny ""
  set c_dicons ""
  set c_icons ""
  set c_licons ""
  set c_lore ""
  set c_name ""
  set c_rarity ""
  set c_set ""
  set c_stats ""
  set c_subtype ""
  set c_text ""
  set c_type ""
  set c_uniq ""
  set invalid 0

  if [regexp $reLocation $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_location c_icons c_licons c_ltxt  c_dicons c_dtxt] then {
    set c_text "$c_ltxt#$c_dtxt"
    set c_class $c_location

    return [showLocation $c_alignment $c_type $c_subtype  $cardSet $c_uniq $c_name $c_rarity $c_location $c_icons $c_licons $c_ltxt  $c_dicons $c_dtxt]

  }  elseif [regexp $reCharacter $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons  c_lore c_text] then {
        return [showCharacter $c_alignment $c_type $c_subtype $cardSet  $c_uniq $c_name $c_rarity $c_destiny  $c_stats $c_class $c_icons $c_lore $c_text ]
  }  elseif [regexp $reInterrupt $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_lore c_text] then {

        return [showInterrupt $c_alignment $c_type $c_subtype $cardSet $c_uniq  $c_name $c_rarity $c_destiny $c_lore $c_text]
        
  }  elseif [regexp $reEffect $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_lore c_text] then {
        return [showEffect $c_alignment $c_type $c_subtype $cardSet $c_uniq  $c_name $c_rarity $c_destiny $c_lore $c_text]
  }  elseif [regexp $reEpicEvent $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_lore c_text] then {
        return [showEpicEvent $c_alignment $c_type $c_subtype $cardSet $c_uniq  $c_name $c_rarity $c_destiny $c_lore $c_text]
  }  elseif [regexp $reJediTest $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_lore c_text] then {
        return [showJediTest $c_alignment $c_type $c_subtype $cardSet $c_uniq  $c_name $c_rarity $c_destiny $c_lore $c_text]
  }  elseif [regexp $reStarship $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons  c_lore c_text] then {
        return [showStarship $c_alignment $c_type $c_subtype $cardSet $c_uniq  $c_name $c_rarity $c_destiny $c_stats $c_class  $c_icons $c_lore $c_text]
  }  elseif [regexp $reVehicle $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons  c_lore c_text] then {
        return [showVehicle $c_alignment $c_type $c_subtype $cardSet $c_uniq  $c_name $c_rarity $c_destiny $c_stats $c_class  $c_icons $c_lore $c_text]
  }  elseif [regexp $reWeapon $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_stats c_icons  c_lore c_text] then {
        return [showWeapon $c_alignment $c_type $c_subtype  $cardSet $c_uniq $c_name $c_rarity $c_destiny $c_stats $c_icons  $c_lore $c_text]
  }  elseif [regexp $reDevice $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_stats c_icons  c_lore c_text] then {
        return [showDevice $c_alignment $c_type $c_subtype  $cardSet $c_uniq $c_name $c_rarity $c_destiny $c_stats $c_icons  $c_lore $c_text]
  }  elseif [regexp $reCreature $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons  c_lore c_text] then {
        return [showCreature $c_alignment $c_type $c_subtype $cardSet  $c_uniq $c_name $c_rarity $c_destiny  $c_stats $c_class $c_icons $c_lore $c_text ]
  }  elseif [regexp $reObjective $raw_card matchstr c_alignment c_type c_subtype  c_set c_uniq c_name c_rarity c_destiny c_stats c_class c_icons  c_lore c_text] then {
        return [showObjective $c_alignment $c_type $c_subtype $cardSet  $c_uniq $c_name $c_rarity $c_destiny  $c_stats $c_class ] } else {
    set invalid 1
  }
}

proc {setOutputRegular} {} {
#
    global outstyle
    set outstyle "Regular"

    write_text "Output style set to Regular\n"
}

proc {setOutputThumb} {} {
#
    global outstyle
    set outstyle "Thumbnailed"

    write_text "Output style set to Thumbnailed\n"
}

proc {showCharSub} {type} {
switch $type {
        a { return "Alien" }
        r { return "Rebel" }
        R { return "Rebel Alien" }
        d { return "Droid" }
        i { return "Imperial" }
        I { return "Imperial Alien" }
        j { return "Jedi Master" }
    }
    return "UNKNOWN($type)"
}

proc {showCharacter} {alignment type subtype set uniq name rarity destiny stats class icons lore text} {
set cardName [eq $name]
    set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny $stats $icons]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] [showCharSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats $class\\n"
    append string "Icons: [eq $icons]\\n\\n"
    append string "Lore: [eq $lore]\\n\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#777"
    return $string
}

proc {showCreature} {alignment type subtype set uniq name rarity destiny stats class icons lore text} {
set cardName [eq $name]
    set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny $stats $icons]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] $class \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats\\n"
    append string "Icons: [eq $icons]\\n\\n"
    append string "Lore: [eq $lore]\\n\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#CCC"
    return $string
}

proc {showDevice} {alignment type subtype set uniq name rarity destiny stats icons lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny $stats $icons]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] Device \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n\\n"
    append string "Lore: [eq $lore]\\n\\n"
    append string "Text: [eq $text]\\n\" "  

    append string "#0C0"
    return $string
}

proc {showEffect} {alignment type subtype set uniq name rarity destiny lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny "" ""]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] [showEffectSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n\\n"
    append string "Lore: [eq $lore]\\n\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#C00"
    return $string
}

proc {showEffectSub} {type} {
switch $type {
          e { return "Effect" }
          i { return "Immediate Effect" }
          m { return "Mobile Effect" }
          u { return "Utinni Effect" }
          t { return "Effect This Is" }
        }
        return ""
}

proc {showEpicEvent} {alignment type subtype set uniq name rarity destiny lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny "" ""]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] Epic Event \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#C0C"
    return $string
}

proc {showIcons} {icons} {
set string " "
    if {[string match "*Pilot*" $icons]} {
        # Yellow
        append string {\016}
    }
    if {[string match "*Astromech*" $icons]} {
        # Red
        append string {\014}
    }
    if {[string match "*Warrior*" $icons]} {
        # Blue
        append string {\018}
    }
    if {[string match "*Scomp*" $icons]} {
        # Grey
        append string {\012}
    }
    if {[string match "*Selective*" $icons]} {
        # Green
        append string {\017}
    }
    return $string
}

proc {showInterSub} {type} {
switch $type {
          l { return "Lost Interrupt" }
          u { return "Used Interrupt" }
          o { return "Used Or Lost Interrupt" }
          O { return "Used Or Starting Interrupt" }
        }
        return ""
}

proc {showInterrupt} {alignment type subtype set uniq name rarity destiny lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny "" ""]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] [showInterSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n\\n"
    append string "Lore: [eq $lore]\\n\\n"
    append string "Text: [eq $text]\\n\\n\" "

    append string "#C00"
    return $string
}

proc {showJediTest} {alignment type subtype set uniq name rarity destiny lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny "" ""]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] Jedi Test #$subtype \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#440"
    return $string
}

proc {showLocSub} {type} {
switch $type {
          s { return "Site" }
          S { return "System" }
          e { return "Sector" }
        }
        return ""
}

proc {showLocation} {alignment type subtype set uniq name rarity location icons licons ltxt dicons dtxt} {
set cardName [eq $name]
    global outstyle

    set string "card \""

    if {$outstyle == "Regular"} {

        append string "$cardName"

        append string "[makePad $cardName]"

        for {set i 0} {$i < 3} {incr i} {
            if {$i < $dicons} {
                # red force
                append string {\009}
            } else {
                # blank
                append string "_"
            }
        }
        for {set i 0} {$i < 3} {incr i} {
            if {$i < $licons} {
                # blue force
                append string {\008}
            } else {
                # blank
                append string "_"
            }
        }
        append string "\" "
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }


    append string "\"[eq $uniq$name]\\n"
    append string "[showSide $alignment] [showLocSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    if {[string length $location]} {
        append string "$location\\n"
    }
    append string "Icons: $icons\\n"
    append string "Light \[$licons\]: [eq $ltxt]\\n"
    append string "Dark \[$dicons\]: [eq $dtxt]\" "

    append string "#000"
    return $string
}

proc {showObjective} {alignment type subtype set uniq name rarity destiny lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string "/TWOSIDED"
        append string [makeTitle $name $destiny "" ""]
    } else {
        append string "/TWOSIDED"
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"$uniq$name ($destiny)\\n"
    append string "[showSide $alignment] Objective \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n\\n"
    set slashLoc [string first "/" $name]

    set SideTitle [string range $name 0 [incr slashLoc -1]]
    append string "$SideTitle:\\n [eq $lore]\\n\\n"
    set SideTitle [string range $name [incr slashLoc 2] end]
    append string "$SideTitle:\\n[eq $text]\\n\" "

    append string "#440"
    return $string
}

proc {showSet} {set} {
switch $set {
        E { return "Empire Strikes Back Introductory Two Player Game" }
        i { return "Premiere Introductory Two Player Game" }
        a { return "A New Hope" }
        A { return "Revised A New Hope" }
        c { return "Cloud City" }
        d { return "Dagobah" }
        D { return "Revised Dagobah" }
        e { return "Endor" }
        P { return "Enhanced Premiere Pack" }
        h { return "Hoth" }
        H { return "Revised Hoth" }
        J { return "Jabba's Palace" }
        j { return "Jedi Pack" }
        p { return "Premiere" }
        U { return "Premiere Unlimited" }
        r { return "Rebel Leader" }
        s { return "Special Edition" }
        S { return "Sealed Deck" }
        z { return "Death Star" }
        l { return "Enhanced Cloud City" }
        b { return "Enhanced Jabba's Palace" }
        f { return "Reflections"}
        w { return "Anthologies"}
        default { return "Unknown \[$set\]\\n" }
    }
}

proc {showSide} {alignment} {
if {$alignment == "d"} {
        return "Dark"
    } else {
        return "Light"
    }
}

proc {showStarSub} {type} {
switch $type {
        c { return "Capitol" }
        f { return "Starfighter" }
        s { return "Squadron" }
    }
    return ""
}

proc {showStarship} {alignment type subtype set uniq name rarity destiny stats class icons lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny $stats $icons]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] [showStarSub $subtype]: $class \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats\\n"
    append string "Icons: [eq $icons]\\n\\n"
    append string "Lore: [eq $lore]\\n\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#00C"
    return $string
}

proc {showText} {label text} {
return "${label}: $text"
}

proc {showVehiSub} {type} {
switch $type {
          c { return "Combat Vehicle: " }
          a { append card "Creature Vehicle" }
          t { append card "Transport Vehicle" }
          s { append card "Shuttle Vehicle" }
        }
        return ""
}

proc {showVehicle} {alignment type subtype set uniq name rarity destiny stats class icons lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny $stats $icons]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] [showVehiSub $subtype]$class \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats\\n"
    append string "Icons: [eq $icons]\\n\\n"
    append string "Lore: [eq $lore]\\n\\n"
    append string "Text: [eq $text]\\n\" "

    append string "#00C"
    return $string
}

proc {showWeapSub} {type} {
switch $type {
          c { return "Character Weapon" }
          a { return "Automated Weapon" }
          s { return "Starship Weapon" }
          v { return "Vehicle Weapon" }
          r { return "Artillery Weapon" }
          d { return "Deathstar Weapon" }
        }
}

proc {showWeapon} {alignment type subtype set uniq name rarity destiny stats icons lore text} {
set string "card \""
    global outstyle

    if {$outstyle == "Regular"} {
        append string [makeTitle $name $destiny $stats $icons]
    } else {
        append string [makeTitleThumb $name $set $alignment]
    }

    append string "\"[eq $uniq$name] ($destiny)\\n"
    append string "[showSide $alignment] [showWeapSub $subtype] \[$rarity\]\\n"
    append string "Set: [showSet $set]\\n"
    append string "$stats\\n\\n"
    append string "Lore: [eq $lore]\\n\\n"
    append string "Text: [eq $text]\\n\" "  

    append string "#0C0"
    return $string
}

proc {wd1ctd} {fp ofp} {
global data1
    global d
    set backImg "blotch.gif"

    set data $data1

    gets $fp hidden
    gets $fp deckData

    regsub -all -- {;} $deckData " " deckData
    regsub -all -- {-} $deckData " " deckData
    
    set length [llength $data]

    foreach {index count} $deckData {
    puts "$index x $count"

        if {$index > $length} {
            write_text "*** $index is out of range!\n"
        } else {
            set card [lindex $data $index]

            regexp {^.;.;.;.;[^;]*;([^;]*);.*} $card matchStr name

            write_text "$count x \"$name\"\n"

            if {[regexp {^d;} $card]} {
                set cardBack "imp.gif"
            } else {
                set cardBack "reb.gif"
            }
            if {$cardBack != $backImg} {
                puts $ofp "back $cardBack"
                set backImg $cardBack
            }

            puts $ofp "[semiToCard $card ""] $count"
        }
    }

    close $fp
    close $ofp
    write_text "\n*** Done converting.\n"
}

proc {wd2ctd} {fp ofp} {
global data
    set backImg "blotch.gif"

    while {! [eof $fp] } {
        set card [gets $fp]
        set reStr {[^;]*}
        set reCard "^($reStr);($reStr);($reStr);($reStr);($reStr)"
        if [regexp $reCard $card matchstr cardName cardSide cardSet cardCount cardVisible] {
            if {!$cardCount} {break}
            write_text "$cardCount x \"$cardName\"\n"
            set index [lsearch -regexp $data "$cardSide;.;.;$cardSet;$reStr;[escapeChars $cardName];.*"]

# Mismatch between Windeck2 and Windeck3 .dat files
            if {$index == -1} {
                set index [lsearch -regexp $data "$cardSide;.;.;[fixSet $cardSet];$reStr;[escapeChars $cardName];.*"]
            }

            if {$index == -1} {
                write_text "* $cardName not found *\n"
            } else {

                if {$cardSide == "d"} {
                    set cardBack "imp.gif"
                } else {
                    set cardBack "reb.gif"
                }

                if {$cardBack != $backImg} {
                    puts $ofp "back $cardBack"
                    set backImg $cardBack
                }

                puts $ofp "[semiToCard [lindex $data $index] $cardSet] $cardCount"
            }
        } else {
#            write_text "$cardSide $cardSet"
        }
    }
    close $fp
    close $ofp
    write_text "\n*** Done converting.\n"
}

proc {wd3ctd} {fp ofp} {
global data
    set backImg "blotch.gif"

    while {! [eof $fp] } {
        set card [gets $fp]
        set reStr {[^;]*}
        set reCard "^($reStr);($reStr);($reStr);($reStr);($reStr);($reStr)"
        if [regexp $reCard $card matchstr cardName cardSide cardSet cardCount cardVisible nullValue] {
            if {!$cardCount} {break}
            write_text "$cardCount x \"$cardName\"\n"
            set index [lsearch -regexp $data "$cardSide;.;.;$cardSet;$reStr;[escapeChars $cardName];.*"]
# Mismatch between Windeck2 and Windeck3 .dat files
            if {$index == -1} {
                set index [lsearch -regexp $data "$cardSide;.;.;[fixSet $cardSet];$reStr;[escapeChars $cardName];.*"]
            }

            if {$index == -1} {
                write_text "* $cardName not found *\n"
            } else {

                if {$cardSide == "d"} {
                    set cardBack "imp.gif"
                } else {
                    set cardBack "reb.gif"
                }

                if {$cardBack != $backImg} {
                    puts $ofp "back $cardBack"
                    set backImg $cardBack
                }

                puts $ofp "[semiToCard [lindex $data $index] $cardSet] $cardCount"
            }
        } else {
#            write_text "$cardSide $cardSet"
        }
    }
    close $fp
    close $ofp
    write_text "\n*** Done converting.\n"
}

proc {wdConvert} {infile outfile} {
global data1
    global data
    global nodats

    set fp [open $infile r]

    if {$nodats == 1} {
        write_text "*** No data files found!  See warning above!\n"
        return
    }

    set ofp [open $outfile w]

    write_text "\n*** Converting...\n"

    gets $fp type

    if {[regexp {^cards\.dat} $type]} {
        write_text "*** This appears to be a WinDeck 1 file...\n"
        if {[llength $data1] == 1} {
            write_text "*** cards.dat wasn't read in properly. Bailing.\n"
        } else {
            wd1ctd $fp $ofp
        }
    } elseif {[regexp {NEEDFILE} $type]} {
        write_text "*** This appears to be a WinDeck 2 file...\n"
        if {[llength $data] == 1} {
            write_text "*** WinDeck 2 data files not read in properly. Bailing.\n"
        } else {
            wd2ctd $fp $ofp
        }
    } elseif {[regexp {GAME} $type]} {
        write_text "*** This appears to be a WinDeck 3 file...\n"
        if {[llength $data] == 1} {
            write_text "*** WinDeck 3 data files not read in properly. Bailing.\n"
        } else {
            wd3ctd $fp $ofp
        }
    } else {
        write_text "*** File format not recognized!\n"
    }
}

proc {write_text} {string} {
#
    .top17.cpd22.03 configure -state normal
    .top17.cpd22.03 insert end "$string"
    .top17.cpd22.03 configure -state disabled
    .top17.cpd22.03 see end
    update idletasks
    return
}

proc {main} {argc argv} {
    global outstyle; 
#RPC
#Added infile/outfile references
    global infile; 
    global outfile; 
    setOutputThumb
    set datdir "."

#Initialize Variables
#If statement for standalone iconified parsing
    set infile ""
    set outfile ""

    if {$argc == 4} {
       set datdir "[lindex $argv 0]"
       set outstyle "[lindex $argv 1]"
       set infile "[lindex $argv 2]"
       set outfile "[lindex $argv 3]"
    } else {
       wm deiconify .top17
       update
    }
#set readData to $datdir instead of "."
    readData $datdir
###
    write_text "\n*** Done reading datafiles\n\n"
#RPC
#Autostart of conversion if parameters passed
    if {($infile != "") && ($outfile != "")} {
	if {$::nodats} {exit 1}
        convertFile
        exit
    }
###
}

proc {Window} {args} {
global vTcl
    set cmd [lindex $args 0]
    set name [lindex $args 1]
    set newname [lindex $args 2]
    set rest [lrange $args 3 end]
    if {$name == "" || $cmd == ""} {return}
    if {$newname == ""} {
        set newname $name
    }
    set exists [winfo exists $newname]
    switch $cmd {
        show {
            if {$exists == "1" && $name != "."} {wm deiconify $name; return}
            if {[info procs vTclWindow(pre)$name] != ""} {
                eval "vTclWindow(pre)$name $newname $rest"
            }
            if {[info procs vTclWindow$name] != ""} {
                eval "vTclWindow$name $newname $rest"
            }
            if {[info procs vTclWindow(post)$name] != ""} {
                eval "vTclWindow(post)$name $newname $rest"
            }
        }
        hide    { if $exists {wm withdraw $newname; return} }
        iconify { if $exists {wm iconify $newname; return} }
        destroy { if $exists {destroy $newname; return} }
    }
}

#################################
# VTCL GENERATED GUI PROCEDURES
#

proc vTclWindow. {base} {
    if {$base == ""} {
        set base .
    }
    ###################
    # CREATING WIDGETS
    ###################
    wm focusmodel $base passive
    wm geometry $base 1x1+0+0
    wm maxsize $base 1137 834
    wm minsize $base 104 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm withdraw $base
    wm title $base "vt.tcl"
    ###################
    # SETTING GEOMETRY
    ###################
}

proc vTclWindow.top17 {base} {
    if {$base == ""} {
        set base .top17
    }
    if {[winfo exists $base]} {
        wm deiconify $base; return
    }
    ###################
    # CREATING WIDGETS
    ###################
    toplevel $base -class Toplevel
    wm iconify $base
    wm focusmodel $base passive
    wm geometry $base 500x400+267+117
    wm maxsize $base 1137 834
    wm minsize $base 104 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
#RPC
#Commented out auto-deiconify
#    wm deiconify $base
###
    wm title $base "Windeck 2 CardTable"
    menubutton $base.cpd23 \
        -anchor nw -menu .top17.cpd23.01 -padx 4 -pady 3 -text File -width 4 
    menu $base.cpd23.01 \
        -activeborderwidth 1 -borderwidth 1 -tearoff 0 
    $base.cpd23.01 add command \
        -command setOutputRegular -label {Output Regular} 
    $base.cpd23.01 add command \
        -command setOutputThumb -label {Output Thumbnails} 
    $base.cpd23.01 add separator
    $base.cpd23.01 add command \
        -command convertFile -label Convert 
    $base.cpd23.01 add separator
    $base.cpd23.01 add command \
        -command exit -label Exit 
    frame $base.cpd22 \
        -borderwidth 1 -height 30 -relief raised -width 30 
    scrollbar $base.cpd22.01 \
        -borderwidth 1 -command {.top17.cpd22.03 xview} -orient horiz \
        -width 10 
    scrollbar $base.cpd22.02 \
        -borderwidth 1 -command {.top17.cpd22.03 yview} -orient vert \
        -width 10 
    text $base.cpd22.03 \
        -font -Adobe-Courier-Medium-R-Normal-*-*-140-*-*-*-*-*-* -height 1 \
        -state disabled -width 8 -xscrollcommand {.top17.cpd22.01 set} \
        -yscrollcommand {.top17.cpd22.02 set} 
    ###################
    # SETTING GEOMETRY
    ###################
    place $base.cpd23 \
        -x 0 -y 0 -relwidth 1 -height -375 -relheight 1 -anchor nw 
    place $base.cpd22 \
        -x 0 -y 25 -width 0 -relwidth 1 -height -25 -relheight 1 -anchor nw 
    grid columnconf $base.cpd22 0 -weight 1
    grid rowconf $base.cpd22 0 -weight 1
    grid $base.cpd22.01 \
        -in .top17.cpd22 -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew 
    grid $base.cpd22.02 \
        -in .top17.cpd22 -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns 
    grid $base.cpd22.03 \
        -in .top17.cpd22 -column 0 -row 0 -columnspan 1 -rowspan 1 \
        -sticky nesw 
}

Window show .
#RPC
#Started window differently so it doesn't show immediately
vTclWindow.top17 ""
###

main $argc $argv
